==> iwrite-1.0.1.4.tar.gz: iwrite-1.0.1.4/src/option.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "option.h"
#include "version.h"

extern void
option_assign(
  struct option*const                   o_option)
{

  memset(o_option, 0, sizeof(*o_option));
  (*o_option).m_prompt= g_hash_table_new(g_str_hash, g_str_equal);

  return;
}

static void
option_discharge_key(
  gpointer                              l_key,
  gpointer                              l_value,
  gpointer                              l_user_data)
{

  g_free(l_key);
  g_free(l_value);

  return;
}

extern void
option_discharge(
  struct option*const                   io_option)
{

  if ((*io_option).m_outputfile)
  {
    g_free((*io_option).m_outputfile);
  }

  if ((*io_option).m_password)
  {
    g_free((*io_option).m_password);
  }

  if ((*io_option).m_reportfile)
  {
    g_free((*io_option).m_reportfile);
  }

  g_hash_table_foreach((*io_option).m_prompt, option_discharge_key, 0);
  g_hash_table_destroy((*io_option).m_prompt);

  memset(io_option, 0, sizeof(*io_option));

  return;
}

static int
option_prompt_add_hash(
  GHashTable*                           io_prompt,
  char *const*                          i_prompt)
{
  int                                   l_exit;
  size_t                                l_slot;
  char*                                 l_key;
  char*                                 l_value;
  char*                                 l_ptr;
  ptrdiff_t                             l_size;

  l_exit= 0;

  do
  {
    if(0 == io_prompt)
    {
      break; 
    }

    for(l_slot= 0; i_prompt[l_slot]; l_slot++)
    {

      l_ptr= strrchr(i_prompt[l_slot], ':');

      if (l_ptr)
      { 
        l_size= (l_ptr - i_prompt[l_slot]);
        l_key= (char*)malloc(1+l_size);
        memset(l_key, 0, 1+l_size);
        memcpy(l_key, i_prompt[l_slot], l_size);

        l_ptr++;
        l_size= strlen(l_ptr);
        l_value= (char*)malloc(1+l_size);
        memset(l_value, 0, 1+l_size);
        memcpy(l_value, l_ptr, l_size);

        g_hash_table_insert(io_prompt, l_key, l_value);

        if (0 == (l_key[0]) || (0 == l_value[0]))
        { 
          g_warning("Missing key or value: %s\n", i_prompt[l_slot]);
          l_exit= -1;
        }

      }
      else
      {
        g_warning("Unable to parse command line: %s\n", i_prompt[l_slot]);
        l_exit= -1;
      }
    }

  }while(0);

  return l_exit;
}

extern int
option_parse(
  struct option*const                   io_option,
  int                                   i_argc,
  char **                               i_argv)
{
  static char*                          l_format;
  static char*                          l_outputfile;
  static char*                          l_password;
  static char**                         l_prompt;
  static char*                          l_reportfile;
  static GOptionEntry                   l_entries[]=
  {
    {"output", 0, 0, G_OPTION_ARG_FILENAME, &l_outputfile, "output file name", "outputfile"},
    {"password", 0, 0, G_OPTION_ARG_STRING, &l_password, "database password", "password"},
    {"prompt", 0, 0, G_OPTION_ARG_STRING_ARRAY, &l_prompt, "query prompt (--prompt=field:value", "field:value"},
    {"report", 0, 0, G_OPTION_ARG_FILENAME, &l_reportfile, "report to run", "reportname"},
/*
    {"format", 0, 0, G_OPTION_ARG_STRING, &l_format, "output format (pdf)", "pdf"},
*/
    {0}
  };
  gboolean                              l_bool;
  GOptionContext*                       l_ctx;
  GError*                               l_error;
  int                                   l_exit;
  char*                                 l_filename;
  char*                                 l_ptr;
  int                                   l_rc;
  unsigned                              l_size;
  unsigned                              l_slot;
  char                                  l_summary[64];

  l_ctx= 0;
  l_error= 0;
  l_exit= 0;
  l_filename= 0;
  l_format= 0;
  l_outputfile= 0;
  l_password= 0;
  l_prompt= 0;
  l_reportfile= 0;

  snprintf(
    l_summary,
    sizeof(l_summary),
    "iwrite - %d.%d.%d.%d (%s%s%s)",
    g_version.m_major,
    g_version.m_minor,
    g_version.m_stage,
    g_version.m_micro,
    g_version.m_stamp.m_year,
    g_version.m_stamp.m_month,
    g_version.m_stamp.m_day);

  do
  {

    l_ctx= g_option_context_new(0);
    g_option_context_add_main_entries(l_ctx, l_entries, 0);
    g_option_context_add_group(l_ctx, gtk_get_option_group(TRUE));
    g_option_context_set_summary(l_ctx, l_summary);
    l_bool= g_option_context_parse(l_ctx, &i_argc, &i_argv, &l_error);

    if (0 == l_bool)
    {
      _error_log(l_error);
      g_warning("Unable to parse command line: %s\n", l_error->message);
      l_exit= -1;
      break;
    }

    if (l_prompt)
    {
      option_prompt_add_hash((*io_option).m_prompt, l_prompt);
    }

    if (0 == l_reportfile)
    {
      if (l_outputfile || l_format)
      {
        g_warning("Report file missing from command line\n");
        l_exit= -1;
      }
      break;
    }

    l_size= strlen(l_reportfile);
    l_filename= g_malloc0(6+l_size);
    strcpy(l_filename, l_reportfile);
    l_ptr= strrchr(l_filename, '.');

    if (0 == l_ptr)
    {
      strcat(l_filename, ".bson");
    }
    else
    {
      l_rc= strncmp(l_ptr, ".bson", 5);
      if (l_rc)
      {
        strcat(l_filename, ".bson");
      }
    }

    l_rc= g_file_test(l_filename, G_FILE_TEST_EXISTS);

    if (0 == l_rc)
    {
      g_warning("File does not exist: %s\n", l_filename);
      l_exit= -1;
      break;
    }

    (*io_option).m_reportfile= l_filename;
    l_filename= 0;

    if (l_outputfile)
    {
      (*io_option).m_outputfile= l_outputfile;
      l_outputfile= 0;
    }

    (*io_option).m_password= l_password;
    l_password= 0;

  }while(0);

  if (l_ctx)
  {
    g_option_context_free(l_ctx);
  }

  g_clear_error(&l_error);

  if (l_filename)
  {
    g_free(l_filename);
  }

  if (l_format)
  {
    g_free(l_format);
  }

  if (l_prompt)
  {
    for(l_slot= 0; l_prompt[l_slot]; l_slot++)
    {
      g_free(l_prompt[l_slot]);
    }
  }

  if (l_outputfile)
  {
    g_free(l_outputfile);
  }

  if (l_reportfile)
  {
    g_free(l_reportfile);
  }

  return l_exit;
}

==> iwrite-3.0.1.0.tar.gz: iwrite-3.0.1.0/src/option.c <==
/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "option.h"
#include "version.h"

extern void
option_assign(
  struct option*const                   o_option)
{

  memset(o_option, 0, sizeof(*o_option));
  (*o_option).m_prompt= g_hash_table_new(g_str_hash, g_str_equal);

  return;
}

static void
option_discharge_key(
  gpointer                              l_key,
  gpointer                              l_value,
  gpointer                              l_user_data)
{

  g_free(l_key);
  g_free(l_value);

  return;
}

extern void
option_discharge(
  struct option*const                   io_option)
{

  if ((*io_option).m_outputfile)
  {
    g_free((*io_option).m_outputfile);
  }

  if ((*io_option).m_password)
  {
    g_free((*io_option).m_password);
  }

  if ((*io_option).m_reportfile)
  {
    g_free((*io_option).m_reportfile);
  }

  g_hash_table_foreach((*io_option).m_prompt, option_discharge_key, 0);
  g_hash_table_destroy((*io_option).m_prompt);

  memset(io_option, 0, sizeof(*io_option));

  return;
}

static int
option_prompt_add_hash(
  GHashTable*                           io_prompt,
  char *const*                          i_prompt)
{
  int                                   l_exit;
  size_t                                l_slot;
  char*                                 l_key;
  char*                                 l_value;
  char*                                 l_ptr;
  ptrdiff_t                             l_size;

  l_exit= 0;

  do
  {
    if(0 == io_prompt)
    {
      break; 
    }

    for(l_slot= 0; i_prompt[l_slot]; l_slot++)
    {

      l_ptr= strrchr(i_prompt[l_slot], ':');

      if (l_ptr)
      { 
        l_size= (l_ptr - i_prompt[l_slot]);
        l_key= (char*)malloc(1+l_size);
        memset(l_key, 0, 1+l_size);
        memcpy(l_key, i_prompt[l_slot], l_size);

        l_ptr++;
        l_size= strlen(l_ptr);
        l_value= (char*)malloc(1+l_size);
        memset(l_value, 0, 1+l_size);
        memcpy(l_value, l_ptr, l_size);

        g_hash_table_insert(io_prompt, l_key, l_value);

        if (0 == (l_key[0]) || (0 == l_value[0]))
        { 
          g_warning("Missing key or value: %s\n", i_prompt[l_slot]);
          l_exit= -1;
        }

      }
      else
      {
        g_warning("Unable to parse command line: %s\n", i_prompt[l_slot]);
        l_exit= -1;
      }
    }

  }while(0);

  return l_exit;
}

extern int
option_parse(
  struct option*const                   io_option,
  int                                   i_argc,
  char **                               i_argv)
{
  static char*                          l_format;
  static char*                          l_outputfile;
  static char*                          l_password;
  static char**                         l_prompt;
  static char*                          l_reportfile;
  static GOptionEntry                   l_entries[]=
  {
    {"output", 0, 0, G_OPTION_ARG_FILENAME, &l_outputfile, "output file name", "outputfile"},
    {"password", 0, 0, G_OPTION_ARG_STRING, &l_password, "database password", "password"},
    {"prompt", 0, 0, G_OPTION_ARG_STRING_ARRAY, &l_prompt, "query prompt (--prompt=field:value", "field:value"},
    {"report", 0, 0, G_OPTION_ARG_FILENAME, &l_reportfile, "report to run", "reportname"},
/*
    {"format", 0, 0, G_OPTION_ARG_STRING, &l_format, "output format (pdf)", "pdf"},
*/
    {0}
  };
  gboolean                              l_bool;
  GOptionContext*                       l_ctx;
  GError*                               l_error;
  int                                   l_exit;
  unsigned                              l_slot;
  char                                  l_summary[64];

  l_ctx= 0;
  l_error= 0;
  l_exit= 0;
  l_format= 0;
  l_outputfile= 0;
  l_password= 0;
  l_prompt= 0;
  l_reportfile= 0;

  snprintf(
    l_summary,
    sizeof(l_summary),
    "iwrite - %d.%d.%d.%d (%s%s%s)",
    g_version.m_major,
    g_version.m_minor,
    g_version.m_stage,
    g_version.m_micro,
    g_version.m_stamp.m_year,
    g_version.m_stamp.m_month,
    g_version.m_stamp.m_day);

  do
  {

    l_ctx= g_option_context_new(0);
    g_option_context_add_main_entries(l_ctx, l_entries, 0);
    g_option_context_add_group(l_ctx, gtk_get_option_group(FALSE));
    g_option_context_set_summary(l_ctx, l_summary);
    l_bool= g_option_context_parse(l_ctx, &i_argc, &i_argv, &l_error);

    if (0 == l_bool)
    {
      _error_log(l_error);
      g_warning("Unable to parse command line: %s\n", l_error->message);
      l_exit= -1;
      break;
    }

    if (l_prompt)
    {
      option_prompt_add_hash((*io_option).m_prompt, l_prompt);
    }

    if (0 == l_reportfile)
    {
      if (l_outputfile || l_format)
      {
        g_warning("Report file missing from command line\n");
        l_exit= -1;
      }
      break;
    }

    (*io_option).m_reportfile= l_reportfile;
    l_reportfile= 0;

    if (l_outputfile)
    {
      (*io_option).m_outputfile= l_outputfile;
      l_outputfile= 0;
    }

    (*io_option).m_password= l_password;
    l_password= 0;

  }while(0);

  if (l_ctx)
  {
    g_option_context_free(l_ctx);
  }

  g_clear_error(&l_error);

  if (l_format)
  {
    g_free(l_format);
  }

  if (l_prompt)
  {
    for(l_slot= 0; l_prompt[l_slot]; l_slot++)
    {
      g_free(l_prompt[l_slot]);
    }
  }

  if (l_outputfile)
  {
    g_free(l_outputfile);
  }

  if (l_reportfile)
  {
    g_free(l_reportfile);
  }

  return l_exit;
}
